@@ -1,6 +1,7 @@
 __docformat__ = 'google'
 
 import icepool
+from icepool.pool.cost import estimate_costs
 
 from abc import ABC, abstractmethod
 from collections import defaultdict
@@ -54,10 +55,12 @@ class EvalPool(ABC):
                 current outcome. If there was no previous outcome, this will be
                 `None`.
             outcome: The current outcome.
-                `next_state` will see all outcomes in monotonic order,
+                `next_state` will see all rolled outcomes in monotonic order;
                 either ascending or descending depending on `direction()`.
                 If there are multiple pools, the set of outcomes is the union of
-                the outcomes of the invididual pools.
+                the outcomes of the invididual pools. Each outcome will be seen
+                at most once; however, outcomes may be skipped if no dice
+                actually rolled that outcome.
             *counts: One `int` for each pool indicating how many dice in that
                 pool rolled the current outcome. If there are multiple pools,
                 it's possible that some outcomes will not appear in all pools.
@@ -100,11 +103,8 @@ class EvalPool(ABC):
     def direction(self, *pools):
         """Optional function to determine the direction in which `next_state()` will see outcomes.
 
-        Note that an ascending (> 0) direction is not compatible with pools with
-        `truncate_min`, and a descending (< 0) direction is not compatible with
-        pools with `truncate_max`.
-
-        The default is ascending order.
+        The default is ascending order. This works well with mixed standard dice,
+        and other dice that differ only by right-truncation.
 
         Args:
             *pools: One or more `Pool`s being evaluated.
@@ -139,39 +139,34 @@ class EvalPool(ABC):
                     Outcomes are treated as having 1 count per appearance.
                 Most evaluators will expect a fixed number of pools.
                 The union of the outcomes of the pools must be totally orderable.
-                Pools with `truncate_min` and pools with `truncate_max` are not
-                compatible with each other.
 
         Returns:
             A die representing the distribution of the final score.
             If all pools are `PoolRoll`s, the result is a single outcome instead.
         """
 
-        # Convert non-pool arguments to `PoolBase`.
+        # Convert non-pool arguments to `PoolInternal`.
         pools = [
-            pool
-            if isinstance(pool, icepool.PoolBase) else icepool.PoolRoll(pool)
+            pool if isinstance(pool, icepool.Pool) else icepool.Pool(*pool)
             for pool in pools
         ]
 
+        if any(pool.has_empty_dice() for pool in pools):
+            return icepool.Die()
+
         algorithm, direction = self._select_algorithm(*pools)
 
         dist = algorithm(direction, *pools)
 
-        if all(pool._is_single_roll() for pool in pools):
-            for state in dist.keys():
-                # Return a single outcome.
-                return state
-        else:
-            final_outcomes = []
-            final_weights = []
-            for state, weight in dist.items():
-                outcome = self.final_outcome(state, *pools)
-                if outcome is not icepool.Reroll:
-                    final_outcomes.append(outcome)
-                    final_weights.append(weight)
+        final_outcomes = []
+        final_weights = []
+        for state, weight in dist.items():
+            outcome = self.final_outcome(state, *pools)
+            if outcome is not icepool.Reroll:
+                final_outcomes.append(outcome)
+                final_weights.append(weight)
 
-            return icepool.Die(*final_outcomes, weights=final_weights)
+        return icepool.Die(*final_outcomes, weights=final_weights)
 
     __call__ = eval
 
@@ -184,37 +179,28 @@ class EvalPool(ABC):
                 1 for ascending and -1 for descending.
 
         """
-        has_truncate_min = any(pool._has_truncate_min() for pool in pools)
-        has_truncate_max = any(pool._has_truncate_max() for pool in pools)
+        eval_direction = self.direction(*pools)
 
-        if has_truncate_min and has_truncate_max:
-            raise ValueError(
-                'A set of pools cannot be evaluated if they have both truncate_min and truncate_max.'
-            )
+        pop_min_costs, pop_max_costs = zip(
+            *(estimate_costs(pool) for pool in pools))
 
-        direction = self.direction(*pools)
+        pop_min_cost = math.prod(pop_min_costs)
+        pop_max_cost = math.prod(pop_max_costs)
 
-        if not direction:
-            if has_truncate_max:
-                direction = 1
-            elif has_truncate_min:
-                direction = -1
-            else:
-                score_ascending = sum(
-                    pool._direction_score_ascending() for pool in pools)
-                score_descending = sum(
-                    pool._direction_score_descending() for pool in pools)
-                if score_ascending < score_descending:
-                    direction = -1
-                else:
-                    direction = 1
-
-        if direction > 0 and has_truncate_min or direction < 0 and has_truncate_max:
-            # Forced onto the less-preferred algorithm.
-            return self._eval_internal_iterative, direction
+        if pop_max_cost <= pop_min_cost:
+            cost_direction = 1
         else:
+            cost_direction = -1
+
+        if eval_direction == 0:
+            return self._eval_internal, cost_direction
+
+        if eval_direction == cost_direction:
             # Use the preferred algorithm.
-            return self._eval_internal, direction
+            return self._eval_internal, eval_direction
+        else:
+            # Forced onto the less-preferred algorithm.
+            return self._eval_internal_iterative, eval_direction
 
     def _eval_internal(self, direction, *pools):
         """Internal algorithm for iterating in the more-preferred direction,
@@ -237,7 +223,7 @@ class EvalPool(ABC):
 
         result = defaultdict(int)
 
-        if all(len(pool.outcomes()) == 0 for pool in pools):
+        if all(pool.is_empty() for pool in pools):
             result = {None: 1}
         else:
             outcome, iterators = _pop_pools(direction, pools)
@@ -259,10 +245,12 @@ class EvalPool(ABC):
 
         This algorithm does not perform persistent memoization.
         """
+        if all(pool.is_empty() for pool in pools):
+            return {None: 1}
         dist = defaultdict(int)
         dist[None, pools] = 1
-        # This is only safe because all pools are guaranteed to pop outcomes at the same rate.
-        while not all(len(pool.outcomes()) == 0 for pool in pools):
+        final_dist = defaultdict(int)
+        while dist:
             next_dist = defaultdict(int)
             for (prev_state, prev_pools), weight in dist.items():
                 # The direction flip here is the only purpose of this algorithm.
@@ -272,9 +260,12 @@ class EvalPool(ABC):
                     prod_weight = math.prod(weights)
                     state = self.next_state(prev_state, outcome, *counts)
                     if state is not icepool.Reroll:
-                        next_dist[state, pools] += weight * prod_weight
+                        if all(pool.is_empty() for pool in pools):
+                            final_dist[state] += weight * prod_weight
+                        else:
+                            next_dist[state, pools] += weight * prod_weight
             dist = next_dist
-        return {state: weight for (state, _), weight in dist.items()}
+        return final_dist
 
 
 def _pop_pools(side, pools):
@@ -286,56 +277,16 @@ def _pop_pools(side, pools):
     """
     if side >= 0:
         outcome = max(
-            pool._max_outcome() for pool in pools if len(pool.outcomes()) > 0)
-        iterators = tuple(_pop_pool_max(outcome, pool) for pool in pools)
+            pool.max_outcome() for pool in pools if not pool.is_empty())
+        iterators = tuple(pool._pop_max(outcome) for pool in pools)
     else:
         outcome = min(
-            pool._min_outcome() for pool in pools if len(pool.outcomes()) > 0)
-        iterators = tuple(_pop_pool_min(outcome, pool) for pool in pools)
+            pool.min_outcome() for pool in pools if not pool.is_empty())
+        iterators = tuple(pool._pop_min(outcome) for pool in pools)
 
     return outcome, iterators
 
 
-def _pop_pool_max(outcome, pool):
-    """Iterates over possible numbers of dice that could roll an outcome.
-
-    Args:
-        outcome: The max outcome.
-        pool: The `Pool` under consideration.
-
-    Yields:
-        prev_pool: The remainder of the pool after taking out the dice that
-            rolled the current outcome.
-        count: How many dice rolled the current outcome, or 0 if the outcome is
-            not in this pool.
-        weight: The weight of that many dice rolling the current outcome.
-    """
-    if outcome not in pool.outcomes():
-        yield pool, 0, 1
-    else:
-        yield from pool._pop_max()
-
-
-def _pop_pool_min(outcome, pool):
-    """Iterates over possible numbers of dice that could roll an outcome.
-
-    Args:
-        outcome: The min outcome.
-        pool: The `Pool` under consideration.
-
-    Yields:
-        prev_pool: The remainder of the pool after taking out the dice that
-            rolled the current outcome.
-        count: How many dice rolled the current outcome, or 0 if the outcome is
-            not in this pool.
-        weight: The weight of that many dice rolling the current outcome.
-    """
-    if outcome not in pool.outcomes():
-        yield pool, 0, 1
-    else:
-        yield from pool._pop_min()
-
-
 class WrapFuncEval(EvalPool):
     """An `EvalPool` created from a single provided function.
 
@@ -418,6 +369,12 @@ class SumPool(EvalPool):
         else:
             return state + outcome * count
 
+    def final_outcome(self, final_state, *pools):
+        if final_state is None:
+            return 0
+        else:
+            return final_state
+
     def direction(self, *pools):
         """This eval doesn't care about direction. """
         return 0
